﻿// Disclaimer:
// This software is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
//
// This file is part of Printer Demo.
//
// Version 1.01 - May 07, 2012
// Supported Devices: IT-9000
//
// Remarks: You need latest versions of patches and runtime
// libraries be installed onto your mobile device.
//////////////////////////////////////////////////////////////////////

using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using CasioEurope.Samples.DataCapture;
using Calib;

// This demo diplays and prints a delivery note including data and signature.
namespace PrinterDemo
{
    public partial class MainForm : Form
    {
        // Printer commands.
        private static readonly string Cancel = new string(new char[] { '\x18' });
        private static readonly string FlushPrinterBuffer = new string(new char[] { '\x1B', '\x45' });
        private static readonly string DoNotContinuePrintingAfterError = new string(new char[] { '\x1B', '\x52', '\x00' });
        private static readonly string Paperfeed = new string(new char[] { '\x1B', '\x42', '\x00', '\x0F' });
        private static readonly string SetFontSizeToMedium = new string(new char[] { '\x1B', '\x53', '\x02', '\x02' });

        // Initializes the user interface.
        public MainForm()
        {
            InitializeComponent();
            Resources.Culture = CultureInfo.CurrentCulture;
            Text = Resources.MainText;
            this.menuNew.Text = Resources.MenuNew;
            this.menuPrint.Text = Resources.MenuPrint;
            this.lblHeader.Text = Resources.LabelHeader;
            this.signBox.Text = Resources.SignBoxText;
            using (Graphics g = CreateGraphics())
            {
                if (g.DpiX > 100)
                    this.signBox.PenWidth = this.signBox.PenWidth * 2;
            }
            PopulateListBox();
        }

        // Generates new delivery note.
        private void menuNew_Click(object sender, EventArgs e)
        {
            this.signBox.Clear();
            this.lbxData.Items.Clear();
            PopulateListBox();
        }

        // Prints the delivery note.
        private void menuPrint_Click(object sender, EventArgs e)
        {
            // Open the driver.
            if (PrinterLibNet.Api.PRNOpen() != PrinterLibNet.Def.PRN_NORMAL)
            {
                CheckPrinterError();
                return;
            }

            // Do not continue printing after error.
            if (PrinterLibNet.Api.PRNTextOut(DoNotContinuePrintingAfterError.Length, DoNotContinuePrintingAfterError) != PrinterLibNet.Def.PRN_NORMAL)
            {
                CheckPrinterError();
                PrinterLibNet.Api.PRNClose();
                return;
            }

            // Print the header.
            if (PrinterLibNet.Api.PRNPrintWindow(this.lblHeader.Handle) != PrinterLibNet.Def.PRN_NORMAL)
            {
                CheckPrinterError();
                PrinterLibNet.Api.PRNClose();
                return;
            }

            // Change font size.
            if (PrinterLibNet.Api.PRNTextOut(SetFontSizeToMedium.Length, SetFontSizeToMedium) != PrinterLibNet.Def.PRN_NORMAL)
            {
                CheckPrinterError();
                PrinterLibNet.Api.PRNClose();
                return;
            }

            // Print all entries from listbox.
            foreach (string s in this.lbxData.Items)
            {
                string line = string.Format("{0}\n", s);
                if (PrinterLibNet.Api.PRNTextOut(line.Length, line) != PrinterLibNet.Def.PRN_NORMAL)
                {
                    CheckPrinterError();
                    PrinterLibNet.Api.PRNClose();
                    return;
                }
            }

            // Print the signature.
            if (PrinterLibNet.Api.PRNPrintWindow(this.signBox.Handle) != PrinterLibNet.Def.PRN_NORMAL)
            {
                CheckPrinterError();
                PrinterLibNet.Api.PRNClose();
                return;
            }

            // Feed the paper.
            if (PrinterLibNet.Api.PRNTextOut(Paperfeed.Length, Paperfeed) != PrinterLibNet.Def.PRN_NORMAL)
            {
                CheckPrinterError();
                PrinterLibNet.Api.PRNClose();
                return;
            }

            // Force the printer to print all data that is in the printer buffer.
            if (PrinterLibNet.Api.PRNTextOut(FlushPrinterBuffer.Length, FlushPrinterBuffer) != PrinterLibNet.Def.PRN_NORMAL)
            {
                CheckPrinterError();
                PrinterLibNet.Api.PRNClose();
                return;
            }

            // Close the driver.
            if (PrinterLibNet.Api.PRNClose() != PrinterLibNet.Def.PRN_NORMAL)
            {
                CheckPrinterError();
            }
        }

        // Check last error and cancel printing.
        private void CheckPrinterError()
        {
            string message;
            int error = PrinterLibNet.Api.PRNGetLastError();

            switch (error)
            {
                case PrinterLibNet.Def.FUNCTION_UNSUPPORT:
                    message = Resources.FUNCTION_UNSUPPORT;
                    break;
                case PrinterLibNet.Def.PRN_ALREADY_OPEN:
                    message = Resources.PRN_ALREADY_OPEN;
                    break;
                case PrinterLibNet.Def.PRN_AUTOLOADING:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_AUTOLOADING;
                    break;
                case PrinterLibNet.Def.PRN_COVER_CLOSE:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_COVER_CLOSE;
                    break;
                case PrinterLibNet.Def.PRN_DECODE_ERROR:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_DECODE_ERROR;
                    break;
                case PrinterLibNet.Def.PRN_DRIVER_NOTEXIST:
                    message = Resources.PRN_DRIVER_NOTEXIST;
                    break;
                case PrinterLibNet.Def.PRN_FEEDKEY_ERROR:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_FEEDKEY_ERROR;
                    break;
                case PrinterLibNet.Def.PRN_FILE_NOTEXIST:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_FILE_NOTEXIST;
                    break;
                case PrinterLibNet.Def.PRN_FILEFORMAT_ERROR:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_FILEFORMAT_ERROR;
                    break;
                case PrinterLibNet.Def.PRN_FILEOPEN_ERROR:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_FILEOPEN_ERROR;
                    break;
                case PrinterLibNet.Def.PRN_HARDWARE_ERROR:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_HARDWARE_ERROR;
                    break;
                case PrinterLibNet.Def.PRN_HEADTEMP_ERROR:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_HEADTEMP_ERROR;
                    break;
                case PrinterLibNet.Def.PRN_NOT_SUPPORTED:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_NOT_SUPPORTED;
                    break;
                case PrinterLibNet.Def.PRN_NOTCHANGE:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_NOTCHANGE;
                    break;
                case PrinterLibNet.Def.PRN_NOTFOUND:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_NOTFOUND;
                    break;
                case PrinterLibNet.Def.PRN_NOTOPEN:
                    message = Resources.PRN_NOTOPEN;
                    break;
                case PrinterLibNet.Def.PRN_PAPER_END:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_PAPER_END;
                    break;
                case PrinterLibNet.Def.PRN_PARAMETER_ERROR:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_PARAMETER_ERROR;
                    break;
                case PrinterLibNet.Def.PRN_PLATEN_OPEN:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_PLATEN_OPEN;
                    break;
                case PrinterLibNet.Def.PRN_SUSPEND_OCCURRED:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_SUSPEND_OCCURRED;
                    break;
                case PrinterLibNet.Def.PRN_VDETP_OCCURRED:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.PRN_VDETP_OCCURRED;
                    break;
                default:
                    PrinterLibNet.Api.PRNTextOut(Cancel.Length, Cancel);
                    message = Resources.ErrorUnknown;
                    break;
            }

            MessageBox.Show(message, Resources.MessageCaption);
        }

        // Fill delivery note with random data.
        private void PopulateListBox()
        {
            Random rdm1 = new Random(unchecked((int)DateTime.Now.Ticks));
            Random rdm2 = new Random(~unchecked((int)DateTime.Now.Ticks));
            int number = rdm1.Next(10, 20);

            this.lbxData.BeginUpdate();
            for (int i = 0; i < number; i++)
            {
                this.lbxData.Items.Add(string.Format(
                    "{0} {1}:  {2} x {3} #{4}", 
                    Resources.ListPosition,
                    (i + 1).ToString("D2"), 
                    rdm1.Next(1, 9), 
                    Resources.ListItem,
                    rdm2.Next(5000, int.MaxValue)));
            }
            this.lbxData.EndUpdate();
        }
    }
}